/**
 * Copyright (c) 2010 Andreas Sewe
 * 
 * Permission to copy, use, modify (only the included source files, modification
 * of the binaries is not permitted) the software is granted provided this
 * copyright notice appears in all copies.
 * 
 * This software is  provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 */
package ch.usi.dag.jp2.dacapo;

import org.dacapo.harness.Callback;
import org.dacapo.harness.CommandLineArgs;

import ch.usi.dag.jp2.runtime.JP2Runtime;

/**
 * A callback to activate profiling only for benchmark proper; JVM startup and
 * shutdown as well as the benchmark harness are ignored.
 * 
 * To use this callback, pass the following command line arguments to the DaCapo
 * benchmark harness:
 * 
 * <code>--callback ch.usi.dag.jp2.dacapo.JP2Callback</code>
 * 
 * @author Andreas Sewe
 * 
 * @see <a href="http://www.dacapobench.org/">DaCapo Benchmark Suite</a>
 */
public class JP2Callback extends Callback {
    
    public JP2Callback(CommandLineArgs args) {
        super(args);
    }
    
    @Override
    public void start(String benchmark) {
        System.err.println("Activating bytecode counting");
        JP2Runtime.enableProfiling();
        super.start(benchmark);
    }
    
    @Override
    public void stop() {
        super.stop();
        JP2Runtime.disableProfiling();
        System.err.println("Deactivated bytecode counting");
    }
}
