package ch.usi.dag.jp2.dump.text;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.List;
import java.util.zip.GZIPOutputStream;

import ch.usi.dag.jp2.runtime.CCTNode;

/**
 * @author Aibek Sarimbekov
 * @author Philippe Moret
 * @author Walter Binder
 * @author Andreas Sewe
 */
public class AlternativeTextDumper extends AbstractTextDumper {
	
    final static IdentityHashMap<String,Integer> dictMap=new IdentityHashMap<String,Integer>();
    final static List<String> dictList = new ArrayList<String>();
	
    public AlternativeTextDumper(String outputFilePrefix) throws IOException {
        final FileOutputStream fos = new FileOutputStream(outputFilePrefix + ".alternative-output.gz");
        final FileOutputStream fos2 = new FileOutputStream(outputFilePrefix + "-dict.output.gz");
        
        final GZIPOutputStream gzfos = new GZIPOutputStream(fos);
        final GZIPOutputStream gzfos2 = new GZIPOutputStream(fos2);
        
        out = new PrintStream(new BufferedOutputStream(gzfos), false, "UTF-8");
        out2 = new PrintStream(new BufferedOutputStream(gzfos2), false, "UTF-8");
        
    }
    
    private final PrintStream out;
    private final PrintStream out2;
    
    public void dump(CCTNode node) {
        out.println('{');
        
        dumpCallees(1, node.getCallees());
        out.println('}');
        out.flush();
        out.close();
        
        dumpTable();
    }
    
    private void dumpTable() {
        Iterator<String> iterator = dictList.iterator();
        
        while (iterator.hasNext()) {  
            String key = iterator.next();  
            String position = dictMap.get(key).toString();  
            final int[] bbSize = CCTNode.bbSizes.get(key);
            StringBuffer buf = new StringBuffer();
            buf.append(position+ "-->"+key);
            buf.append(Arrays.toString(bbSize));
            out2.println(buf);
        }  
        
        out2.flush();
        out2.close();
        
    }
    
    @Override
    public String getMethodEntry(CCTNode n) {
        final String mid = n.getMID();
        
        int uniqueID = returnMethodUniqueId(mid);
        StringBuffer buf = new StringBuffer(256);
        buf.append("@");
        buf.append(n.getBytecodeIndex());
        buf.append(" "+uniqueID);
        buf.append("=");
        buf.append(n.getExecutionCount());
        buf.append(",");
        buf.append(n.getExecutedInstructions());
        
        buf.append(n.getBBCountsAsString());
       
        return buf.toString();
    }
    
    private static int returnMethodUniqueId(String mid) {
        if (dictMap.containsKey(mid)) {
            return dictMap.get(mid);
        } else {
            int currentPosition = dictMap.size();
            dictMap.put(mid, currentPosition);
            dictList.add(mid);
            return currentPosition;
        }
    }
}
