package ch.usi.dag.jp2.dump.text;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.List;
import java.util.Arrays;
import java.util.Set;
import java.util.TreeSet;
import java.util.zip.GZIPOutputStream;

import ch.usi.dag.jp2.runtime.CCTNode;

/**
 * @author Aibek Sarimbekov
 * @author Philippe Moret
 * @author Walter Binder
 * @author Andreas Sewe
 */
public class TextDumper extends AbstractTextDumper {
	public static long methodCounter=0; //total number of nodes
	public static long execmethodCounter=0; //total number of nodes

    public static long bbCounts=0; //total number of executed basic blocks
    public static long bytecodes=0; //total number of executed bytecodes
    public static Set<String>methods = new TreeSet<String>(); //Set containing the unique nodes
    
    public TextDumper(String outputFilePrefix) throws IOException {
        final FileOutputStream fos = new FileOutputStream(outputFilePrefix + ".output.gz");
        final GZIPOutputStream gzfos = new GZIPOutputStream(fos);
        out = new PrintStream(new BufferedOutputStream(gzfos), false, "UTF-8");
    }
    
    public void dump(CCTNode node) {
        out.println('{');
        
        dumpCallees(1, node.getCallees());
        out.println('}');
        out.flush();
        out.close();
    }
    
    @Override
    public String getMethodEntry(CCTNode n) {
        if (n.getExecutionCount()!=0 || n.getMID().contains("[NATIVE]")) 
            methodCounter++;
        execmethodCounter+=n.getExecutionCount();
    	List<Integer> bbSize = n.getBasicBlockSizes();
        List<Long> bbs = n.getBasicBlockExecutionCounts();
        int[][] bbInst = n.getBasicBlockInst(); // Wing: Get the basic block instructions
        int[] nanoPattern = n.getNanoPattern(); // Wing: Get the nano-pattern
        String sourceFileName = n.getSourceFileName(); // Wing: get the source file name
        
        StringBuffer buf = new StringBuffer(n.getMID());
        if (n.getExecutionCount()!=0 || n.getMID().contains("[NATIVE]")) 
            addUniqueNode(n.getMID());
        buf.append(" = ");
        buf.append(n.getExecutionCount());
        //bytecodes+=n.getExecutionCount();
        buf.append(", ");
        buf.append(n.getExecutedInstructions());
        buf.append(" @");
        buf.append(n.getBytecodeIndex());
        
        buf.append(" [");
        for (int i=0; i < bbs.size(); i++){
            buf.append(bbs.get(i));
            bbCounts+=bbs.get(i);
            buf.append("x" + bbSize.get(i));
            bytecodes+=bbSize.get(i)*bbs.get(i);
            if(i != bbs.size() - 1)
                buf.append(", ");
        }
        buf.append("] ");
        // Wing: Some methods have no instructions
        if (bbInst != null)
        {
            // Wing: for each basic block in the method
            for (int i = 0; i < bbInst.length; i++)
            {
                // Wing: append the basic block number and it's bytecode
                buf.append(i + ": " + Arrays.toString(bbInst[i]) + " ");
            }
        }
        // Wing: append the nano-pattern
        buf.append("np : " + Arrays.toString(nanoPattern) + " ");
        buf.append("\"" + sourceFileName + "\" ");
        return buf.toString();
    }
    static void addUniqueNode(String s){
    	if(methods.contains(s))
    		return;
    	else
    		methods.add(s);
    }
}
