package uk.ac.glasgow.jsinger.nanopatterns;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.commons.EmptyVisitor;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.MethodNode;

public class PatternSpotter {

	public int[] getMethodNanoPatterns(String className, MethodNode method)
	{
		RecursivePatternSpotter rps = new RecursivePatternSpotter(new EmptyVisitor(),
			className, method.name, method.desc);
		OOAccessPatternSpotter ops = new OOAccessPatternSpotter(new EmptyVisitor());
		TypeManipulatorPatternSpotter tps = new TypeManipulatorPatternSpotter(new EmptyVisitor());
		ControlFlowPatternSpotter cps = new ControlFlowPatternSpotter(new EmptyVisitor());
		ArrayAccessPatternSpotter aps = new ArrayAccessPatternSpotter(new EmptyVisitor());
		PolymorphicPatternSpotter pps = new PolymorphicPatternSpotter(new EmptyVisitor());
		ReturnPatternSpotter retps = new ReturnPatternSpotter(new EmptyVisitor());
		MethodPatternSpotter mps = new MethodPatternSpotter(new EmptyVisitor());
		// check following
		// properties directly from
		// method descriptor
		boolean noParams = false;
		boolean noReturn = false;
		boolean throwsExceptions = false;
		if (method.desc.startsWith("()")) {
		    noParams = true;
		}
		if (method.desc.endsWith(")V")) {
		    noReturn = true;
		}
		if (method.exceptions.size() > 0) {
		    throwsExceptions = true;
		}
		if (method.instructions.size() > 0) 
		{
			for (int j = 0; j < method.instructions.size(); ++j) 
			{
				Object insn = method.instructions.get(j);
				((AbstractInsnNode) insn).accept(rps);
				((AbstractInsnNode) insn).accept(ops);
				((AbstractInsnNode) insn).accept(tps);
				((AbstractInsnNode) insn).accept(cps);
				((AbstractInsnNode) insn).accept(aps);
				((AbstractInsnNode) insn).accept(pps);
				((AbstractInsnNode) insn).accept(retps);
				((AbstractInsnNode) insn).accept(mps);
			}
		}
		int [] nanoPatternVector = {
			booleanToInt(noParams),
			booleanToInt(noReturn),
			booleanToInt(rps.isRecursive()),
			booleanToInt(rps.isSameNameCaller()),
			booleanToInt(rps.isLeaf()),
			booleanToInt(ops.isObjectCreator()),
			booleanToInt(ops.isThisInstanceFieldReader()),
			booleanToInt(ops.isThisInstanceFieldWriter()),
			booleanToInt(ops.isOtherInstanceFieldReader()),
			booleanToInt(ops.isOtherInstanceFieldWriter()),
			booleanToInt(ops.isStaticFieldReader()),
			booleanToInt(ops.isStaticFieldWriter()),
			booleanToInt(tps.isTypeManipulator()),
			booleanToInt(cps.isStraightLineCode()),
			booleanToInt(cps.isLoopingCode()),
			booleanToInt(cps.isSwitcher()),
			booleanToInt(throwsExceptions),
			booleanToInt(aps.isLocalVarReader()),
			booleanToInt(aps.isLocalVarWriter()),
			booleanToInt(aps.isArrayCreator()),
			booleanToInt(aps.isArrayReader()),
			booleanToInt(aps.isArrayWriter()),
			booleanToInt(pps.isPolymorphic()),
			booleanToInt(retps.isSingleReturner()),
			booleanToInt(retps.isMultipleReturner()),
			booleanToInt(mps.isClient()),
			booleanToInt(mps.isJdkClient()),
			booleanToInt(mps.isTailCaller())
		    	};
		return nanoPatternVector;
	}
		
	/**
     * trivial support for MP-tool style reporting
     * of exhibited nanopatterns
     */
    public int booleanToInt(boolean value) 
    {
    	return value? 1 : 0;
    }
}
